;
;                       Q U I C K  -  V I E W
;                        by  Andreas Ess 1997
;
;Arguments: OP2 holds thingy to show(not used as it should be used)
;
#include "usgard.h"

.org 0
.db "Q-View 1.0",0

Count           = TEXT_MEM

Start:
	call    CLEARLCD
	ld      (CURSOR_ROW), DE
	
	ld      HL, OP2
	call    VAR_GET
	ret     c

	ld      A, (HL)
	and     A
	ret     nz
	inc     HL
	ld      A, (HL)
	inc     HL
	cp      $FA
	jr      nz, GoOnTest
	ld      HL, &Error
	call    D_ZT_STR
	call    OTH_PAUSE
	ret
GoOnTest:
	cp      $7D
	jr      z, BWPic
	cp      $7E
	jr      z, GR4Pic
	cp      $7F
	jr      z, GR8Pic
	cp      $80
	jr      z, ZCPPic
	cp      $81
	jp      z, &GCPPic
	cp	$83
	jp	z, &WAVFile
	ret

;-- B/W picture -------------------------------------------------------------
BWPic:
	ld      DE, VIDEO_MEM
	ld      BC, 1024
	ldir
	call    &WaitContr
	ret

;-- 4 grayscale picture -----------------------------------------------------
GR4Pic:
	push    HL
	call    &InitGray4
	pop     HL
	ret     c
	ld      DE, VIDEO_MEM
	ld      BC, 1024
	ldir
	ld      DE, (Page2Addr)
	ld      BC, 1024
	ldir
WaitExitGray:
	call    &WaitContr
	call    &GrayRemove
	ret

;-- 8 grayscale picture -----------------------------------------------------
GR8Pic:
	push    HL
	call    &InitGray8
	pop     HL
	ret     c
	ld      DE, VIDEO_MEM
	ld      BC, 1024
	ldir
	ld      DE, (Page2Addr)
	ld      BC, 2048
	ldir
	jr      WaitExitGray

;-- ZCP compressed picture --------------------------------------------------
ZCPPic:
	ld      DE, VIDEO_MEM
	#fncall ZCP
	call    &WaitContr
	ret

;-- GCP compressed picture --------------------------------------------------
GCPPic:
	push    HL
	call    &InitGray4
	pop     HL
	ret     c
	ld      BC, 0
	#fncall GCP
	jr      WaitExitGray

WAVFile:
	push	HL
	ld	HL, $0101
	ld	(CURSOR_X), HL
	ld	HL, &WAV
	call	D_ZM_STR
	pop	HL
	di                       ;Disable interrupts -> can't use GET_KEY
	ld    a, 0C0h            ;first, send a 0 bit to link port.
	out   (7), a
	dec	HL
	dec	HL
	dec	HL
	dec	HL
	ld    	E, (HL)            ;read size from the first 2 bytes at the WAVE data
	inc   	HL                 ;lower byte read
	ld    	D, (HL)            ;read high byte
	inc   	HL                 ;the following bytes contains the WAV information
	ld    	B, 10000000b       ;B is used as a bit-mask
SoundLoop:
	ld    	C, 17              ;delay some microseconds
DLoop:
	dec   	C                  ;C = C - 1
	jr    	nz, DLoop          ;if C isn't 0 -> jump to DLoop again
	rrc   	B                  ;rotate B: 76543210 -> 07654321(in our case:
			  ;e.g. 10000000b -> 01000000b
			  ;if   00000001b -> 10000000b, then the CARRY FLAG
			  ;is set.
	jr    	nc, Play           ;if no carry(carry -> next byte), play normally
	inc   	HL                 ;next byte
	dec   	DE                 ;decrement the size
	ld    	A, D               ;if it gets zero -> quit
	or    	E                  ;make OR D, E(if both are 0 -> ZERO FLAG SET)
	jr    	z, DonePlay        ;if ZERO FLAG set -> quit
	xor   	a                  ;XOR A is FASTER than LD A, 0
	out   	(1), a             ;write 0=A to keyboard port -> enable all keys
	in    	a, (1)             ;get key pressed -> A
	bit   	6, a               ;if bit 6 of A(EXIT) is pressed, then ZERO FLAG SET
	jr    	z, DonePlay        ;if the ZERO FLAG is set -> quit
Play:                     ;normal play
	ld    	a, (hl)            ;get byte
	and   	b                  ;and that byte A with the mask B:
			  ;e.g. A = 11001010b and B = 10000000b -> AND
			  ;     A = 10000000b -> ZERO FLAG NOT SET
			  ;     A = 11001010b and B = 00100000b -> AND
				 ;     A = 00000000b -> ZERO FLAG SET
	jr    	z, Play0           ;if bit is zero, send a 0 bit to the link port
	ld    	a, 0FCh            ;send a one bit
	out   	(7), a
	jr    	SoundLoop          ;loop again
Play0:
	ld    	a, 0C0h            ;send a 0 bit
	out   	(7), a
	jr    	SoundLoop          ;loop again
DonePlay:
	ld   a, 0C0h             ;a 0 bit has to be outputted last!
	out  (7), a
	ei                       ;re-enable interrupts
	ret                      ;back to caller -> the main program in our case

InitGray4:
	ld      C, 4            ;4 * 256 bytes pages
	call    &AllocBuf
	ret     c
	ld      HL, &Gray4
	jr      InstallInt

InitGray8:
	ld      C, 8
	call    &AllocBuf
	ret     c
	ld      HL, &Gray8
InstallInt:
	call    INT_INSTALL
	ret

GrayRemove:
	call    INT_REMOVE
	ld      A, $7C
	out     (0), A
	ret

AllocBuf:
	ld      HL, ($8BE5)
	dec     HL
	dec     HL
	ld      A, H
	sub     C
	ld      H, A
	ld      L, 0
	push    HL

	ld      DE, ($8BE1)
	or      A
	sbc     HL, DE
	pop     HL
	ret     c

	and     %11000000
	cp      %11000000
	scf
	ret     nz

	ld      BC, 1024
	ld      (Page2Addr), HL
	ld      A, H
	ld      (&Page2+1), A
	ld      (&Page2_8+1), A
	call    OTH_CLEAR
	ld      A, H
	ld      (&Page3_8+1), A
	or      A
	ret

Gray4:
	ld     HL, Count               ;3
	ld     A, (HL)                 ;1
	cp     2                       ;2
	jr     z, Disp_2               ;2
Disp_1:
	inc    (HL)                    ;1
	ld     A, $7C                  ;2
	out    (0), A                  ;2
	ret                            ;1
Disp_2:
Page2:
	ld     A, $7C                  ;2
	out    (0), A                  ;2
	ld     (HL), 0                 ;2
	ret                            ;1 = 21 bytes

Gray8:
	ld     HL, Count
	ld     A, (HL)
	cp     6
	jr     z, Disp_3_8
	cp     4
	jr     nc, Disp_2_8
Disp_1_8:
	inc    (HL)
Page1_8:
	ld     A, $7C
	out    (0), A
	ret
Disp_2_8:
Page2_8:
	ld     A, $7C
	out    (0), A
	inc    (HL)
	ret
Disp_3_8:
Page3_8:
	ld    A, $3C
	out   (0), A
	ld    (HL), 0
	ret

WaitContr:
	ld      A, (CONTRAST)
	ld      B, A
	ld      C, 2
KeyLoop:
	halt
	call    GET_KEY
	and     a
	jr      z, KeyLoop
	cp      K_ENTER
	jr      nz, NotEnter
	ld      A, B
	ld      (CONTRAST), A
	ret
NotEnter:
	cp      K_EXIT
	jr      nz, NotExit
	ld      A, (CONTRAST)
	out     (2), A
	ret
NotExit:
	cp      K_UP
	jr      z, CUp
	cp      K_DOWN
	jr      nz, WaitContr
CDown:
	ld      A, B
	and     A
	jr      z, KeyLoop
	dec     B
	out     (C), B
	jr      KeyLoop
CUp:
	ld      A, B
	cp      31
	jr      nc, KeyLoop
	inc     B
	out     (C), B
	jr      KeyLoop

Error:
 .db "Don",39,"t call directly!",0
WAV:
 .db "Playing WAV",0

.end
